--[[
ClosetGnome, a World of Warcraft addon to manage item sets.
Modified in WoW 3.1 to just be a LDB plugin, handle keybindings and display the quips.
Copyright (C) 2010 Rabbit.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

local ClosetGnome = CreateFrame("Frame")
_G.ClosetGnome = ClosetGnome

-- see "Blizzard_Deprecated.lua" for 10.0.0
local InterfaceOptions_AddCategory = InterfaceOptions_AddCategory;

if (not InterfaceOptions_AddCategory) then
	InterfaceOptions_AddCategory = function(frame, addOn, position)
		-- cancel is no longer a default option. May add menu extension for this.
		frame.OnCommit = frame.okay;
		frame.OnDefault = frame.default;
		frame.OnRefresh = frame.refresh;

		if frame.parent then
			local category = Settings.GetCategory(frame.parent);
			local subcategory, layout = Settings.RegisterCanvasLayoutSubcategory(category, frame, frame.name, frame.name);
			subcategory.ID = frame.name;
			return subcategory, category;
		else
			local category, layout = Settings.RegisterCanvasLayoutCategory(frame, frame.name, frame.name);
			category.ID = frame.name;
			Settings.RegisterAddOnCategory(category);
			return category;
		end
	end
end

local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory;

if (not InterfaceOptionsFrame_OpenToCategory) then
	InterfaceOptionsFrame_OpenToCategory = function(categoryIDOrFrame)
		if type(categoryIDOrFrame) == "table" then
			local categoryID = categoryIDOrFrame.name;
			return Settings.OpenToCategory(categoryID);
		else
			return Settings.OpenToCategory(categoryIDOrFrame);
		end
	end
end

-- Just so older closetgnome macros will still work for people.
function ClosetGnome:WearSet(s)
	if s == nil then return end
	local equipmentSetId = C_EquipmentSet.GetEquipmentSetID(s)
	C_EquipmentSet.UseEquipmentSet(equipmentSetId)
end
ClosetGnome:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

local db = nil
local L = LibStub("AceLocale-3.0"):GetLocale("ClosetGnome")
local icon = LibStub("LibDBIcon-1.0", true)
local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
local defaultIcon = "Interface\\Icons\\INV_Chest_Cloth_17"
local menuNeedsUpdate = nil

local CGLDB = nil
if ldb then
	CGLDB = ldb:NewDataObject("ClosetGnome", {
		type = "data source",
		text = "ClosetGnome",
		icon = defaultIcon,
	})
end

-------------------------------------------------------------------------------
-- Keybindings
--

local cgKeyFrame = CreateFrame("Button", "ClosetGnomeKeyFrame", UIParent)
cgKeyFrame:SetScript("OnMouseUp", function(self, set)
	local equipmentSetId = C_EquipmentSet.GetEquipmentSetID(set)
	C_EquipmentSet.UseEquipmentSet(equipmentSetId)
end)

-- Verify that the given equipment set actually exists.
local function setExists(set)
	local equipmentSetId = C_EquipmentSet.GetEquipmentSetID(set)
	return equipmentSetId ~= nil
end

local function clearBinding(set)
	if not set or not db.keybindings[set] or not setExists(set) then return end
	SetOverrideBinding(cgKeyFrame, false, db.keybindings[set])
	db.keybindings[set] = nil
	menuNeedsUpdate = true
end

local function registerBinding(combo, set)
	if not combo or not set or not setExists(set) then return end
	if db.keybindings[set] then
		SetOverrideBinding(cgKeyFrame, false, db.keybindings[set])
	end
	db.keybindings[set] = combo
	SetOverrideBindingClick(cgKeyFrame, false, combo, "ClosetGnomeKeyFrame", set)
	menuNeedsUpdate = true
end

-------------------------------------------------------------------------------
-- Config UI
--

local updateKeybindingButtons = nil
do
	local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer or SettingsCanvas)
	frame.name = "ClosetGnome"
	frame:Hide()

	local buttons = {}
	local label = "%s: %s"
	local function updateButton(button)
		local set = button.set
		if db.keybindings[set] then
			button:SetNormalFontObject("GameFontHighlight")
			button:SetText(label:format(set, db.keybindings[set]))
			button:SetAlpha(1)
		else
			button:SetNormalFontObject("GameFontNormal")
			button:SetText(label:format(set, NOT_BOUND))
			button:SetAlpha(0.8)
		end
	end

	local function clearButton(button)
		if not button:IsKeyboardEnabled() then return end
		button:EnableKeyboard(false)
		button:UnlockHighlight()
	end

	local function onClick(self, b)
		if b ~= "LeftButton" then return end
		for i, button in next, buttons do
			if button ~= self then
				clearButton(button)
			end
		end
		if self:IsKeyboardEnabled() then
			clearButton(self)
		else
			self:EnableKeyboard(true)
			self:LockHighlight()
		end
	end

	local ignoreKeys = {
		UNKNOWN = true,
		LSHIFT = true, LCTRL = true, LALT = true,
		RSHIFT = true, RCTRL = true, RALT = true,
	}
	local function onKeyDown(self, key)
		if key == "ESCAPE" then
			key = nil
		else
			if ignoreKeys[key] then return end
			if IsShiftKeyDown() then
				key = "SHIFT-"..key
			end
			if IsControlKeyDown() then
				key = "CTRL-"..key
			end
			if IsAltKeyDown() then
				key = "ALT-"..key
			end
		end
		self:EnableKeyboard(false)
		self:UnlockHighlight()

		if not key or key == "" then
			clearBinding(self.set)
		else
			registerBinding(key, self.set)
		end
		updateButton(self)
	end

	local function createScrollButtons(parent, num)
		for i = 1, num do
			local button = CreateFrame("Button", "CGScrollButton" .. i, parent, "UIPanelButtonTemplate")
			button:SetScript("OnClick", onClick)
			button:SetScript("OnKeyDown", onKeyDown)
			button:SetHeight(25)
			button.tooltipText = L["Keybinding"]
			button.newbieText = L["Click and press a key combination to bind it to this equipment set. Escape clears an existing binding."]
			if i == 1 then
				button:SetPoint("TOPLEFT", parent, 12, -8)
			else
				button:SetPoint("TOPLEFT", buttons[i - 1], "BOTTOMLEFT", 0, -2)
				button:SetPoint("TOPRIGHT", buttons[i - 1], "BOTTOMRIGHT", 0, -2)
			end
			buttons[i] = button
		end
	end

	local function newCheckbox(label, onClick)
		local check = CreateFrame("CheckButton", "ClosetGnomeCheck"..label, frame, "InterfaceOptionsCheckButtonTemplate")
		check:SetScript("OnClick", function(self)
			PlaySound(self:GetChecked() and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_OFF)
			onClick(self, self:GetChecked() and true or false)
		end)
		local fs = _G[check:GetName() .. "Text"]
		fs:SetText(label)
		return check
	end

	frame:SetScript("OnShow", function(frame)
		local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
		title:SetPoint("TOPLEFT", 16, -16)
		title:SetText("ClosetGnome")

		local subTitleWrapper = CreateFrame("Frame", nil, frame)
		subTitleWrapper:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
		subTitleWrapper:SetPoint("RIGHT", -16, 0)
		local subtitle = subTitleWrapper:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		subtitle:SetPoint("TOPLEFT", subTitleWrapper)
		subtitle:SetWidth(subTitleWrapper:GetRight() - subTitleWrapper:GetLeft())
		subtitle:SetJustifyH("LEFT")
		subtitle:SetNonSpaceWrap(false)
		subtitle:SetJustifyV("TOP")
		subtitle:SetText(L.closetgnome_description)
		subTitleWrapper:SetHeight(subtitle:GetHeight())

		local quips = newCheckbox(
			L["Quips"],
			function(self, value) ClosetGnome.db.profile.quips = value end)
		quips:SetChecked(ClosetGnome.db.profile.quips)
		quips:SetPoint("TOPLEFT", subTitleWrapper, "BOTTOMLEFT", -2, -16)

		local minimap = nil
		if icon and CGLDB and icon:IsRegistered("ClosetGnome") then
			minimap = newCheckbox(
				L["Show minimap icon"],
				function(self, value)
					local hide = not value
					ClosetGnome.db.profile.minimap.hide = hide
					if hide then
						icon:Hide("ClosetGnome")
					else
						icon:Show("ClosetGnome")
					end
				end)
			minimap:SetChecked(not ClosetGnome.db.profile.minimap.hide)
			minimap:SetPoint("TOPLEFT", quips, "BOTTOMLEFT", 0, -8)
		end

		local bindings = CreateFrame("Frame", "CGKeyBinds", frame, "CGOptionsBoxTemplate")
		bindings:SetPoint("TOPLEFT", minimap or quips, "BOTTOMLEFT", 0, -16)
		bindings:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -16, 12)

		local displayNum = math.floor(bindings:GetHeight() / 25) - 1
		createScrollButtons(bindings, displayNum)

		local scroll = CreateFrame("ScrollFrame", "CGKBScroll", bindings, "FauxScrollFrameTemplateLight")
		scroll:SetPoint("TOPLEFT", bindings, "TOPLEFT", 5, -6)
		scroll:SetPoint("BOTTOMRIGHT", bindings, "BOTTOMRIGHT", -28, 4)

		local tmp = {}
		local function update()
			wipe(tmp)
			for i = 0, C_EquipmentSet.GetNumEquipmentSets() - 1 do
				local name = C_EquipmentSet.GetEquipmentSetInfo(i)
				tmp[#tmp + 1] = name
			end
			local offset = FauxScrollFrame_GetOffset(scroll)
			local index = nil
			for i = 1, displayNum do
				index = offset + i
				if index <= #tmp then
					buttons[i].set = tmp[index]
					updateButton(buttons[i])
					buttons[i]:Show()
				else
					buttons[i]:Hide()
				end
				clearButton(buttons[i])
			end
			if FauxScrollFrame_Update(scroll, #tmp, displayNum, 25) then
				buttons[1]:SetPoint("TOPRIGHT", bindings, -29, -7)
			else
				buttons[1]:SetPoint("TOPRIGHT", bindings, -12, -7)
			end
		end
		scroll:SetScript("OnVerticalScroll", function(self, offset)
			FauxScrollFrame_OnVerticalScroll(self, offset, 25, update)
		end)

		update()

		updateKeybindingButtons = update
		frame:SetScript("OnShow", update)

		newCheckbox = nil
		createScrollButtons = nil
	end)
	InterfaceOptions_AddCategory(frame)
end

local NUM_QUIPS = L["num_quips"]
local function getQuip(input)
	local rand = 1
	repeat rand = math.random(1, NUM_QUIPS) until L[rand] ~= nil
	return "|cffeda55f"..L[rand]:format("|cffd9d919"..input.."|cffeda55f").."|r"
end

--------------------------------------------------------------------------------
-- Utility
--

-- Returns the setId of the currently equipped set, or nil if none matchs.
local function getEquippedSet()
	for i = 0, C_EquipmentSet.GetNumEquipmentSets() - 1 do
		local _, _, setId, isEquipped = C_EquipmentSet.GetEquipmentSetInfo(i)
		if isEquipped then
			return setId
		end
	end
	return nil
end

local function openOptions()
	InterfaceOptionsFrame_OpenToCategory("ClosetGnome")
end

--------------------------------------------------------------------------------
-- Events
--

function ClosetGnome:ADDON_LOADED(addon)
	if addon:lower() ~= "closetgnome" then return end
	self:UnregisterEvent("ADDON_LOADED")

	self.db = LibStub("AceDB-3.0"):New("ClosetGnomeDB", {
		profile = {
			quips = true,
			minimap = {
				hide = false,
			},
		},
		char = {
			keybindings = {},
		},
	}, true)
	db = self.db.char

	SlashCmdList.ClosetGnome = openOptions
	SLASH_ClosetGnome1 = "/closetgnome"
	SLASH_ClosetGnome2 = "/cg"

	-- We don't even remove keybindings when a player
	-- deletes a set. If he later re-adds a set with the same
	-- name, chances are real good he wants the same keybinding
	-- anyway.
	--hooksecurefunc("C_EquipmentSet.DeleteEquipmentSet", clearBinding)

	if icon and CGLDB then
		icon:Register("ClosetGnome", CGLDB, self.db.profile.minimap)
	end

	-- AddonLoader support
	if IsLoggedIn() then self:PLAYER_LOGIN() end
end

function ClosetGnome:EQUIPMENT_SETS_CHANGED()
	menuNeedsUpdate = true
	if updateKeybindingButtons then updateKeybindingButtons() end
end

function ClosetGnome:PLAYER_LOGIN()
	-- Create the button frames for keybindings
	-- Hopefully the Blizzard equipment set information is available at this point.
	for k, v in pairs(db.keybindings) do
		registerBinding(v, k)
	end

	local setId = getEquippedSet()
	if setId then self:EQUIPMENT_SWAP_FINISHED(true, setId, true) end
	self:EQUIPMENT_SETS_CHANGED()
end

function ClosetGnome:EQUIPMENT_SWAP_FINISHED(success, setId, silent)
	if not success then return end
	if CGLDB then
		if setId then
			local name, icon = C_EquipmentSet.GetEquipmentSetInfo(setId)
			CGLDB.text = name
			-- Work around a strange Blizzard API inconsistency where
			-- C_EquipmentSet.GetEquipmentSetInfo only returns the actual icon name, NOT
			-- prepended by Interface\\Icons\\.
			if icon and type(icon) == 'string' and icon:sub(1, 9) ~= "Interface" then icon = "Interface\\Icons\\" .. icon end -- gru: added "and type(icon) == 'string'"
			CGLDB.icon = icon or defaultIcon
		else
			CGLDB.text = "ClosetGnome"
			CGLDB.icon = defaultIcon
		end
	end

	if not silent and setId and ClosetGnome.db.profile.quips then
		local name = C_EquipmentSet.GetEquipmentSetInfo(setId)
		print(getQuip(name))
	end
end

--------------------------------------------------------------------------------
-- LDB Plugin
--

if CGLDB then
	local popupFrame = CreateFrame("Frame", "ClosetGnomeMenu", UIParent, "UIDropDownMenuTemplate")
	local menu = {}

	local function menuSorter(a, b)
		return a.text > b.text
	end

	local function updateMenu()
		menuNeedsUpdate = nil
		menu = wipe(menu)
		for i = 0, C_EquipmentSet.GetNumEquipmentSets() - 1 do
			local name, icon = C_EquipmentSet.GetEquipmentSetInfo(i)
			local k = name
			if db.keybindings[k] then
				name = name .. " |cff999999(" .. db.keybindings[k] .. ")|r"
			end
			menu[#menu + 1] = {
				text = name,
				notCheckable = 1,
				tooltipTitle = NORMAL_FONT_COLOR_CODE .. k .. FONT_COLOR_CODE_CLOSE,
				tooltipText = L["%sClick to equip or Ctrl-Click to delete %s.|r"]:format(HIGHLIGHT_FONT_COLOR_CODE, k),
				tooltipOnButton = 1,
				func = function()
					if IsControlKeyDown() then
						C_EquipmentSet.DeleteEquipmentSet(i)
					else
						C_EquipmentSet.UseEquipmentSet(i)
					end
				end,
				icon = icon or nil,
			}
		end
		table.sort(menu, menuSorter)
	end

	function CGLDB.OnClick(self, button)
		if button == "RightButton" then
			openOptions()
		else
			if menuNeedsUpdate then updateMenu() end
			EasyMenu(menu, popupFrame, self, 20, 4, "MENU")
		end
	end

	function CGLDB.OnTooltipShow(tt)
		tt:AddLine("ClosetGnome")
		tt:AddLine(L.ldbTooltip, 0.2, 1, 0.2, 1)
	end

end

--------------------------------------------------------------------------------
-- Register events
--

ClosetGnome:RegisterEvent("ADDON_LOADED")
ClosetGnome:RegisterEvent("EQUIPMENT_SETS_CHANGED")
ClosetGnome:RegisterEvent("PLAYER_LOGIN")
ClosetGnome:RegisterEvent("EQUIPMENT_SWAP_FINISHED")
